@@ -42,7 +42,7 @@
 #define  I_(a)  a
 #endif
 
-#if  ! ((GTK_MAJOR_VERSION >= 2) && (GTK_MINOR_VERSION >= 14))
+#if  (GTK_MAJOR_VERSION == 1) || ((GTK_MAJOR_VERSION == 2) && (GTK_MINOR_VERSION < 14))
 #define  gtk_adjustment_get_upper(adj)  ((adj)->upper)
 #define  gtk_adjustment_get_value(adj)  ((adj)->value)
 #define  gtk_adjustment_get_page_size(adj)  ((adj)->page_size)
@@ -83,6 +83,16 @@ struct _VteTerminalPrivate
 	x_screen_scroll_event_listener_t  screen_scroll_listener ;
 	int8_t  adj_value_changed_by_myself ;
 
+	/*
+	 * Keep 'is_visible' value of vte_terminal_set_visible_bell() and
+	 * 'is_audible' value of vte_terminal_set_audible_bell() to prepare for
+	 * vte_terminal_set_audible_bell( FALSE ) and
+	 * vte_terminal_set_visible_bell( FALSE ),
+	 * because mlterm doesn't keep values of both visible and sound.
+	 */
+	int8_t  audible_bell ;
+	int8_t  visible_bell ;
+
 	GIOChannel *  io ;
 	guint  src_id ;
 
@@ -153,6 +163,37 @@ static char *  false = "false" ;
 
 /* --- static functions --- */
 
+#if  (GTK_MAJOR_VERSION == 1) || ((GTK_MAJOR_VERSION == 2) && (GTK_MINOR_VERSION < 12))
+/* gdk_color_to_string() was not supported by gtk+ < 2.12. */
+gchar *
+gdk_color_to_string(
+	const GdkColor *  color
+	)
+{
+	gchar *  str ;
+
+	if( ( str = g_malloc( 14)) == NULL)
+	{
+		return  NULL ;
+	}
+
+	sprintf( str , "#%04x%04x%04x" , color->red , color->green , color->blue) ;
+
+	return  str ;
+}
+#endif
+
+static int
+is_initial_allocation(
+	GtkAllocation *  allocation
+	)
+{
+	/* { -1 , -1 , 1 , 1 } is default value of GtkAllocation. */
+	return  (allocation->x == -1 && allocation->y == -1 &&
+		 allocation->width == 1 && allocation->height == 1) ;
+}
+
+
 static void
 catch_child_exited(
 	VteReaper *  reaper ,
@@ -307,16 +348,34 @@ __exit(
 	)
 {
 #ifdef  KIK_DEBUG
-	VteTerminal *  terminal ;
-	
+	u_int  count ;
+
 #if  1
 	kik_mem_dump_all() ;
 #endif
 
-	terminal = p ;
+	ml_free_word_separators() ;
+
+	/*
+	 * Don't loop from 0 to dis.num_of_roots owing to processing inside x_display_remove_root.
+	 */
+	for( count = disp.num_of_roots ; count > 0 ; count--)
+	{
+		if( IS_MLTERM_SCREEN( disp.roots[count - 1]))
+		{
+			gtk_widget_destroy(
+				GTK_WIDGET( VTE_WIDGET((x_screen_t*)disp.roots[count - 1]))) ;
+		}
+		else
+		{
+			x_display_remove_root( &disp , disp.roots[count - 1]) ;
+		}
+	}
+	free( disp.roots) ;
+	x_gc_delete( disp.gc) ;
+	x_xim_display_closed( disp.display) ;
+	x_picture_display_closed( disp.display) ;
 
-	gtk_widget_destroy( GTK_WIDGET(terminal)) ;
-	
 	ml_term_manager_final() ;
 	kik_locale_final() ;
 	x_main_config_final( &main_config) ;
@@ -324,15 +383,18 @@ __exit(
 	x_shortcut_final( &shortcut) ;
 	x_termcap_final( &termcap) ;
 	x_xim_final() ;
-	x_imagelib_display_closed( disp.display) ;
 
 	kik_alloca_garbage_collect() ;
 
 	kik_msg_printf( "reporting unfreed memories --->\n") ;
 	kik_mem_free_all() ;
 #endif
-	
+
+#if  1
+	exit(1) ;
+#else
 	gtk_main_quit() ;
+#endif
 }
 
 
@@ -808,7 +870,7 @@ vte_terminal_filter(
 		{
 			static pid_t  config_menu_pid = 0 ;
 
-			if( ! terminal || /* SCIM etc window */ 
+			if( ! terminal || /* SCIM etc window */
 			    /* XFilterEvent in x_window_receive_event. */
 			    ((XEvent*)xevent)->xany.window != disp.roots[count]->my_window)
 			{
@@ -1024,7 +1086,7 @@ update_wall_picture(
 	}
 
 end:
-	sprintf( file , "pixmap:%d" , terminal->pvt->pixmap) ;
+	sprintf( file , "pixmap:%lu" , terminal->pvt->pixmap) ;
 
 	vte_terminal_set_background_image_file( terminal , file) ;
 }
@@ -1120,9 +1182,7 @@ vte_terminal_realize(
 	 * to x_window_t or ml_term_t, so x_window_resize must be called here.
 	 */
 	if( VTE_TERMINAL(widget)->pvt->term->pty &&
-	   /* { -1 , -1 , 1 , 1 } is default value of GdkAllocation. */
-	   (widget->allocation.x != -1 || widget->allocation.y != -1 ||
-	    widget->allocation.width != 1 || widget->allocation.height != 1) )
+	    ! is_initial_allocation( &widget->allocation))
 	{
 		if( x_window_resize_with_margin( &VTE_TERMINAL(widget)->pvt->screen->window ,
 			widget->allocation.width , widget->allocation.height , NOTIFY_TO_MYSELF))
@@ -1130,7 +1190,7 @@ vte_terminal_realize(
 			reset_vte_size_member( VTE_TERMINAL(widget)) ;
 		}
 	}
-	
+
 	update_wall_picture( VTE_TERMINAL(widget)) ;
 }
 
@@ -1366,6 +1426,8 @@ vte_terminal_class_init(
 		}
 	}
 #endif
+	/* Default value of vte "audible-bell" is TRUE, while "visible-bell" is FALSE. */
+	main_config.bel_mode = BEL_SOUND ;
 
 	kik_conf_delete( conf) ;
 
@@ -1679,6 +1741,7 @@ vte_terminal_init(
 	x_font_manager_t *  font_man ;
 	mkf_charset_t  usascii_font_cs ;
 	int  usascii_font_cs_changable ;
+	gdouble  dpi ;
 
 	GTK_WIDGET_SET_FLAGS( terminal , GTK_CAN_FOCUS) ;
 
@@ -1728,6 +1791,17 @@ vte_terminal_init(
 		usascii_font_cs_changable = 1 ;
 	}
 
+	/* related to x_font_use_point_size_for_xft(1) in vte_terminal_class_init. */
+	if( ( dpi = gdk_screen_get_resolution(
+		gtk_widget_get_screen( GTK_WIDGET(terminal)))) != -1)
+	{
+	#ifdef  DEBUG
+		kik_debug_printf( KIK_DEBUG_TAG " Setting dpi %f\n" , dpi) ;
+	#endif
+
+		x_font_set_dpi_for_xft( dpi) ;
+	}
+
 	font_man = x_font_manager_new( disp.display ,
 			main_config.type_engine ,
 			main_config.font_present ,
@@ -1799,11 +1873,24 @@ vte_terminal_init(
 	terminal->pvt->pix_height = 0 ;
 	terminal->pvt->pic_mod = NULL ;
 
+	/* audible_bell and visible_bell member is initialized by settings of main_config. */
+	terminal->pvt->audible_bell = (main_config.bel_mode == BEL_SOUND) ;
+	terminal->pvt->visible_bell = (main_config.bel_mode == BEL_VISUAL) ;
+
 	terminal->window_title = ml_term_window_name( terminal->pvt->term) ;
 	terminal->icon_title = ml_term_icon_name( terminal->pvt->term) ;
 
+#if  0
 	gtk_widget_ensure_style( &terminal->widget) ;
-	
+#else
+	/*
+	 * XXX
+	 * I don't know why, but gtk_widget_ensure_style() doesn't apply "inner-border"
+	 * and min width/height of roxterm are not correctly set.
+	 */
+	gtk_widget_set_rc_style( &terminal->widget) ;
+#endif
+
 	reset_vte_size_member( terminal) ;
 }
 
@@ -1889,7 +1976,8 @@ vte_terminal_fork_command(
 
 		vte_reaper_add_child( ml_term_get_child_pid( terminal->pvt->term)) ;
 
-		if( GTK_WIDGET_REALIZED(GTK_WIDGET(terminal)))
+		if( GTK_WIDGET_REALIZED(GTK_WIDGET(terminal)) &&
+		    ! is_initial_allocation( &GTK_WIDGET(terminal)->allocation))
 		{
 			if( x_window_resize_with_margin( &terminal->pvt->screen->window ,
 				GTK_WIDGET(terminal)->allocation.width ,
@@ -1974,7 +2062,8 @@ vte_terminal_forkpty(
 
 		vte_reaper_add_child( ml_term_get_child_pid( terminal->pvt->term)) ;
 		
-		if( GTK_WIDGET_REALIZED(GTK_WIDGET(terminal)))
+		if( GTK_WIDGET_REALIZED(GTK_WIDGET(terminal)) &&
+		    ! is_initial_allocation( &GTK_WIDGET(terminal)->allocation))
 		{
 			if( x_window_resize_with_margin( &terminal->pvt->screen->window ,
 				GTK_WIDGET(terminal)->allocation.width ,
@@ -2151,16 +2240,20 @@ vte_terminal_set_audible_bell(
 	gboolean is_audible
 	)
 {
+	terminal->pvt->audible_bell = (is_audible == TRUE) ;
+
 	if( is_audible)
 	{
-		main_config.bel_mode = BEL_SOUND ;
+		x_screen_set_config( terminal->pvt->screen , NULL , "bel_mode" , "sound") ;
+	}
+	else if( terminal->pvt->visible_bell)
+	{
+		x_screen_set_config( terminal->pvt->screen , NULL , "bel_mode" , "visual") ;
 	}
 	else
 	{
-		main_config.bel_mode = BEL_VISUAL ;
+		x_screen_set_config( terminal->pvt->screen , NULL , "bel_mode" , "none") ;
 	}
-	
-	x_screen_set_config( terminal->pvt->screen , NULL , "bel_mode" , "sound") ;
 }
 
 gboolean
@@ -2184,16 +2277,9 @@ vte_terminal_set_visible_bell(
 	gboolean  is_visible
 	)
 {
-	if( is_visible)
-	{
-		main_config.bel_mode = BEL_VISUAL ;
-	}
-	else
-	{
-		main_config.bel_mode = BEL_SOUND ;
-	}
+	terminal->pvt->visible_bell = (is_visible == TRUE) ;
 
-	x_screen_set_config( terminal->pvt->screen , NULL , "bel_mode" , "visual") ;
+	vte_terminal_set_audible_bell( terminal , ! is_visible && terminal->pvt->audible_bell) ;
 }
 
 gboolean
@@ -2257,8 +2343,6 @@ vte_terminal_set_color_foreground(
 	const GdkColor *  foreground
 	)
 {
-	/* gdk_color_to_string() was not supported. */
-#if  (GTK_MAJOR_VERSION >= 2) && (GTK_MINOR_VERSION >= 12)
 	gchar *  str ;
 
 	if( ! foreground)
@@ -2273,9 +2357,6 @@ vte_terminal_set_color_foreground(
 	kik_debug_printf( KIK_DEBUG_TAG " set_color_foreground %s\n" , str) ;
 #endif
 
-	free( main_config.fg_color) ;
-	main_config.fg_color = strdup( str) ;
-
 	if( GTK_WIDGET_REALIZED(GTK_WIDGET(terminal)))
 	{
 		x_screen_set_config( terminal->pvt->screen , NULL , "fg_color" , str) ;
@@ -2286,7 +2367,6 @@ vte_terminal_set_color_foreground(
 	}
 
 	g_free( str) ;
-#endif
 }
 
 void
@@ -2295,8 +2375,6 @@ vte_terminal_set_color_background(
 	const GdkColor *  background
 	)
 {
-	/* gdk_color_to_string() was not supported. */
-#if  (GTK_MAJOR_VERSION >= 2) && (GTK_MINOR_VERSION >= 12)
 	gchar *  str ;
 
 	if( ! background)
@@ -2311,12 +2389,14 @@ vte_terminal_set_color_background(
 	kik_debug_printf( KIK_DEBUG_TAG " set_color_background %s\n" , str) ;
 #endif
 
-	free( main_config.bg_color) ;
-	main_config.bg_color = strdup( str) ;
-	
 	if( GTK_WIDGET_REALIZED(GTK_WIDGET(terminal)))
 	{
 		x_screen_set_config( terminal->pvt->screen , NULL , "bg_color" , str) ;
+
+		if( terminal->pvt->image && terminal->pvt->pic_mod->alpha < 255)
+		{
+			update_wall_picture( terminal) ;
+		}
 	}
 	else
 	{
@@ -2324,7 +2404,6 @@ vte_terminal_set_color_background(
 	}
 
 	g_free( str) ;
-#endif
 }
 
 void
@@ -2333,8 +2412,6 @@ vte_terminal_set_color_cursor(
 	const GdkColor *  cursor_background
 	)
 {
-	/* gdk_color_to_string() was not supported. */
-#if  (GTK_MAJOR_VERSION >= 2) && (GTK_MINOR_VERSION >= 12)
 	gchar *  str ;
 
 	if( ! cursor_background)
@@ -2344,14 +2421,11 @@ vte_terminal_set_color_cursor(
 
 	/* #rrrrggggbbbb */
 	str = gdk_color_to_string( cursor_background) ;
-	
+
 #ifdef  __DEBUG
 	kik_debug_printf( KIK_DEBUG_TAG " set_color_cursor %s\n" , str) ;
 #endif
 
-	free( main_config.cursor_bg_color) ;
-	main_config.cursor_bg_color = strdup( str) ;
-
 	if( GTK_WIDGET_REALIZED(GTK_WIDGET(terminal)))
 	{
 		x_screen_set_config( terminal->pvt->screen , NULL , "cursor_bg_color" , str) ;
@@ -2362,7 +2436,6 @@ vte_terminal_set_color_cursor(
 	}
 
 	g_free( str) ;
-#endif
 }
 
 void
@@ -2382,6 +2455,57 @@ vte_terminal_set_colors(
 	glong palette_size
 	)
 {
+	if( palette_size != 0 && palette_size != 8 && palette_size != 16 &&
+	    ( palette_size < 24 || 256 < palette_size))
+	{
+	#ifdef  DEBUG
+		kik_debug_printf( KIK_DEBUG_TAG " palette_size %d is illegal\n" , palette_size) ;
+	#endif
+
+		return ;
+	}
+
+	if( palette_size >= 8)
+	{
+		ml_color_t  color ;
+		x_color_config_t *  color_config ;
+		int  need_redraw = 0 ;
+
+		if( foreground == NULL)
+		{
+			foreground = &palette[7] ;
+		}
+		if( background == NULL)
+		{
+			background = &palette[0] ;
+		}
+
+		color_config = terminal->pvt->screen->color_man->color_cache->color_config ;
+
+		for( color = 0 ; color < palette_size ; color++)
+		{
+			gchar *  rgb ;
+			char *  name ;
+
+			rgb = gdk_color_to_string( palette + color) ;
+			name = ml_get_color_name( color) ;
+
+		#ifdef  DEBUG
+			kik_debug_printf( KIK_DEBUG_TAG " Setting rgb %s=%s\n" , name , rgb) ;
+		#endif
+
+			need_redraw |= x_customize_color_file( color_config , name , rgb , 0) ;
+
+			g_free( rgb) ;
+		}
+
+		if( need_redraw && GTK_WIDGET_REALIZED(GTK_WIDGET(terminal)))
+		{
+			x_color_cache_unload_all() ;
+			x_screen_reset_view( terminal->pvt->screen) ;
+		}
+	}
+
 	vte_terminal_set_color_foreground( terminal , foreground) ;
 	vte_terminal_set_color_background( terminal , background) ;
 }
@@ -2396,9 +2520,14 @@ vte_terminal_set_default_colors(
 void
 vte_terminal_set_background_image(
 	VteTerminal *  terminal ,
-	GdkPixbuf *  image		/* can be NULL */
+	GdkPixbuf *  image		/* can be NULL and same as current terminal->pvt->image */
 	)
 {
+	if( terminal->pvt->image == image)
+	{
+		return ;
+	}
+	
 	if( terminal->pvt->image)
 	{
 		g_object_unref( terminal->pvt->image) ;
@@ -2429,18 +2558,26 @@ vte_terminal_set_background_image_file(
 	kik_debug_printf( KIK_DEBUG_TAG " Setting image file %s\n" , path) ;
 #endif
 
-	free( main_config.pic_file_path) ;
-	main_config.pic_file_path = strdup( path) ;
-	
+	if( terminal->pvt->image &&
+	    /*
+	     * Don't unref terminal->pvt->image if path is
+	     * "pixmap:<pixmap id>" (Ex. the case of vte_terminal_set_background_image_file()
+	     * being called from update_wall_picture().)
+	     */
+	    (strlen(path) <= 7 || strncmp( path , "pixmap:" , 7) != 0))
+	{
+		g_object_unref( terminal->pvt->image) ;
+		terminal->pvt->image = NULL ;
+	}
+
 	if( GTK_WIDGET_REALIZED(GTK_WIDGET(terminal)))
 	{
-		x_screen_set_config( terminal->pvt->screen , NULL ,
-			"wall_picture" , main_config.pic_file_path) ;
+		x_screen_set_config( terminal->pvt->screen , NULL , "wall_picture" , path) ;
 	}
 	else
 	{
 		free( terminal->pvt->screen->pic_file_path) ;
-		terminal->pvt->screen->pic_file_path = strdup( main_config.pic_file_path) ;
+		terminal->pvt->screen->pic_file_path = strdup( path) ;
 	}
 }
 
@@ -2496,7 +2633,7 @@ vte_terminal_set_opacity(
 	)
 {
 	u_int8_t  alpha ;
-	
+
 	alpha = 255 - ((opacity >> 8) & 0xff) ;
 
 	/* XXX (roxterm always sets opacity 0xffff. roxterm changes saturation instead.) */
@@ -2509,7 +2646,7 @@ vte_terminal_set_opacity(
 	{
 		char  value[DIGIT_STR_LEN(u_int8_t)] ;
 
-		sprintf( value , "%d" , alpha) ;
+		sprintf( value , "%d" , (int)alpha) ;
 
 		x_screen_set_config( terminal->pvt->screen , NULL , "alpha" , value) ;
 		update_wall_picture( terminal) ;
