@@ -1,5 +1,5 @@
 /*
- *	$Id: kik_path.h,v 1.5 2002/02/02 09:20:27 arakiken Exp $
+ *	$Id$
  */
 
 #ifndef   __KIK_PATH_H__
@@ -25,7 +25,11 @@ char *  __kik_basename( char *  path) ;
 #endif
 
 
-int  kik_path_cleanname( char *  cleaned_path , size_t  size , char *  path) ;
+#ifndef  REMOVE_FUNCS_MLTERM_UNUSE
+
+int  kik_path_cleanname( char *  cleaned_path , size_t  size , const char *  path) ;
+
+#endif
 
 
 #endif
