@@ -1,5 +1,5 @@
 /*
- *	$Id: ml_font.h,v 1.2 2004/12/07 03:37:00 h_minami Exp $
+ *	$Id$
  */
 
 #ifndef  __ML_FONT_H__
@@ -9,13 +9,13 @@
 #include  <mkf/mkf_charset.h>
 
 
-#define  DEFAULT_FONT(cs)  (cs)
+#define  NORMAL_FONT_OF(cs)  (cs)
 #define  FONT_CS(font)  ((font) & MAX_CHARSET)
 
 
 typedef enum ml_font
 {
-	/* 0x00 - MAX_CHARSET(0x7ff) is reserved for mkf_charset_t */
+	/* 0x00 - MAX_CHARSET(0x2ff) is reserved for mkf_charset_t */
 
 	/* 0x1000 is reserved for unicode half or full width tag */
 	FONT_BIWIDTH = 0x1000u ,	/* (default) half width */
