@@ -95,6 +95,7 @@ update(
     mc_update_flag_mode(MC_FLAG_MCOL);
     mc_update_flag_mode(MC_FLAG_BIDI);
     mc_update_flag_mode(MC_FLAG_AWIDTH);
+    mc_update_flag_mode(MC_FLAG_CLIPBOARD);
 
     mc_flush(io) ;
 
@@ -582,6 +583,11 @@ show(void)
 	gtk_widget_show(config_widget);
 	gtk_box_pack_start(GTK_BOX(vbox), config_widget, FALSE, FALSE, 0);
 
+	if (!(config_widget = mc_flag_config_widget_new(MC_FLAG_CLIPBOARD)))
+	    return 0;
+	gtk_widget_show(config_widget);
+	gtk_box_pack_start(GTK_BOX(vbox), config_widget, FALSE, FALSE, 0);
+
 
 	gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_MOUSE);
 	gtk_window_set_policy(GTK_WINDOW(window), 0, 0, 0);
