@@ -51,8 +51,6 @@
 #define  MAX_SCREENS  (MSU * max_screens_multiple)	/* Default MAX_SCREENS is 32. */
 #define  MSU          (8 * sizeof(dead_mask[0]))	/* MAX_SCREENS_UNIT */
 
-#define MAX_ADDTIONAL_FDS  3
-
 #if  0
 #define  __DEBUG
 #endif
@@ -90,7 +88,8 @@ static struct
 	int  fd ;
 	void (*handler)( void) ;
 
-} additional_fds[MAX_ADDTIONAL_FDS] ;
+} * additional_fds ;
+static u_int  num_of_additional_fds ;
 
 
 /* --- static functions --- */
@@ -1598,7 +1597,7 @@ receive_next_event(void)
 			}
 		}
 
-		for( count = 0 ; count < MAX_ADDTIONAL_FDS ; count++)
+		for( count = 0 ; count < num_of_additional_fds ; count++)
 		{
 			if( additional_fds[count].fd >= 0)
 			{
@@ -1620,6 +1619,14 @@ receive_next_event(void)
 		{
 			x_display_idling( displays[count]) ;
 		}
+		
+		for( count = 0 ; count < num_of_additional_fds ; count++)
+		{
+			if( additional_fds[count].fd < 0)
+			{
+				(*additional_fds[count].handler)() ;
+			}
+		}
 	}
 	
 	if( ret < 0)
@@ -1669,7 +1676,7 @@ receive_next_event(void)
 		}
 	}
 
-	for( count = 0 ; count < MAX_ADDTIONAL_FDS ; count++)
+	for( count = 0 ; count < num_of_additional_fds ; count++)
 	{
 		if( additional_fds[count].fd >= 0)
 		{
@@ -1701,8 +1708,8 @@ x_term_manager_init(
 	int  use_xim ;
 	u_int  min_font_size ;
 	u_int  max_font_size ;
-	int  count ;
 	char *  invalid_msg = "%s %s is not valid.\n" ;
+	char *  true = "true" ;
 
 	if( ! x_color_config_init( &color_config))
 	{
@@ -1739,6 +1746,15 @@ x_term_manager_init(
 	
 	x_prepare_for_main_config( conf) ;
 
+	/*
+	 * Following options are not possible to specify as arguments of mlclient.
+	 * 1) Options which are used only when mlterm starts up and which aren't
+	 *    changed dynamically. (e.g. "startup_screens")
+	 * 2) Options which change status of all ptys or windows. (Including ones
+	 *    which are possible to change dynamically.)
+	 *    (e.g. "button3_behavior")
+	 */
+
 	kik_conf_add_opt( conf , '@' , "screens" , 0 , "startup_screens" ,
 		"number of screens to open in start up [1]") ;
 	kik_conf_add_opt( conf , 'h' , "help" , 1 , "help" ,
@@ -1785,6 +1801,8 @@ x_term_manager_init(
 		"[menu1]"
 	#endif
 		) ;
+	kik_conf_add_opt( conf , '\0' , "clip" , 1 , "use_clipboard" ,
+		"use CLIPBOARD (not only PRIMARY) selection. [false]") ;
 
 	if( ! kik_conf_parse_args( conf , &argc , &argv))
 	{
@@ -1881,7 +1899,7 @@ x_term_manager_init(
 
 	if( ( value = kik_conf_get_value( conf , "compose_dec_special_font")))
 	{
-		if( strcmp( value , "true") == 0)
+		if( strcmp( value , true) == 0)
 		{
 			x_compose_dec_special_font() ;
 		}
@@ -1889,7 +1907,7 @@ x_term_manager_init(
 
 #ifdef  USE_TYPE_XFT
 	if( ( value = kik_conf_get_value( conf , "use_cp932_ucs_for_xft")) == NULL ||
-		strcmp( value , "true") == 0)
+		strcmp( value , true) == 0)
 	{
 		ml_use_cp932_ucs_for_xft() ;
 	}
@@ -1992,6 +2010,14 @@ x_term_manager_init(
 		x_set_button3_behavior( value) ;
 	}
 
+	if( ( value = kik_conf_get_value( conf , "use_clipboard")))
+	{
+		if( strcmp( value , true) == 0)
+		{
+			x_set_clipboard_selection( 1) ;
+		}
+	}
+
 
 	if( ( version = kik_conf_get_version( conf)) == NULL)
 	{
@@ -2053,12 +2079,6 @@ x_term_manager_init(
 	
 	kik_alloca_garbage_collect() ;
 
-	for( count = 0 ; count < MAX_ADDTIONAL_FDS ; count++)
-	{
-		additional_fds[count].fd = -1 ;
-		additional_fds[count].handler = NULL ;
-	}
-
 	return  1 ;
 }
 
@@ -2082,6 +2102,8 @@ x_term_manager_final(void)
 
 	free( dead_mask) ;
 
+	free( additional_fds) ;
+
 	ml_term_manager_final() ;
 
 	x_display_close_all() ;
@@ -2203,7 +2225,8 @@ x_term_manager_event_loop(void)
 
 						#ifdef  __DEBUG
 							kik_debug_printf( KIK_DEBUG_TAG
-								" closing screen %d.") ;
+								" closing screen %d-%d." ,
+								idx , count) ;
 						#endif
 
 							screen = screens[idx * MSU + count] ;
@@ -2265,31 +2288,41 @@ x_term_manager_event_loop(void)
 	}
 }
 
+/*
+ * fd >= 0  -> Normal file descriptor. handler is invoked if fd is ready.
+ * fd < 0 -> Special ID. handler is invoked at interval of 0.1 sec.
+ */
 int
 x_term_manager_add_fd(
 	int  fd ,
 	void  (*handler)(void)
 	)
 {
-	int  i ;
+	void *  p ;
 
 	if( ! handler)
 	{
-		return  1 ;
+		return  0 ;
 	}
 
-	for( i = 0 ; i < MAX_ADDTIONAL_FDS ; i++)
+	if( ( p = realloc( additional_fds ,
+			sizeof(*additional_fds) * (num_of_additional_fds + 1))) == NULL)
 	{
-		if( additional_fds[i].fd == -1)
-		{
-			additional_fds[i].fd = fd ;
-			additional_fds[i].handler = handler ;
-			kik_file_set_cloexec( fd);
-
-			return  0 ;
-		}
+		return  0 ;
 	}
 
+	additional_fds = p ;
+	additional_fds[num_of_additional_fds].fd = fd ;
+	additional_fds[num_of_additional_fds++].handler = handler ;
+	if( fd >= 0)
+	{
+		kik_file_set_cloexec( fd) ;
+	}
+
+#ifdef  DEBUG
+	kik_debug_printf( KIK_DEBUG_TAG " %d is added to additional fds.\n" , fd) ;
+#endif
+
 	return  1 ;
 }
 
@@ -2298,19 +2331,22 @@ x_term_manager_remove_fd(
 	int  fd
 	)
 {
-	int  i ;
+	u_int  count ;
 
-	for( i = 0 ; i < MAX_ADDTIONAL_FDS ; i++)
+	for( count = 0 ; count < num_of_additional_fds ; count++)
 	{
-		if( additional_fds[i].fd == fd)
+		if( additional_fds[count].fd == fd)
 		{
-			additional_fds[i].fd = -1 ;
-			additional_fds[i].handler = NULL ;
+		#ifdef  DEBUG
+			kik_debug_printf( KIK_DEBUG_TAG " Additional fd %d is removed.\n" , fd) ;
+		#endif
+		
+			additional_fds[count] = additional_fds[--num_of_additional_fds] ;
 
-			return  0 ;
+			return  1 ;
 		}
 	}
 
-	return  1 ;
+	return  0 ;
 }
 
