@@ -34,7 +34,7 @@ typedef int  XIMStyle ;		/* dummy */
 typedef HANDLE XID ;
 typedef HANDLE Window ;
 typedef HDC Drawable ;
-typedef HBITMAP Pixmap ;
+typedef HDC Pixmap ;
 typedef HDC GC ;
 typedef HFONT Font ;
 typedef HCURSOR Cursor ;
